"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var frontmatterTransformer = function (original) {
    var trimed = original.trim();
    if (trimed.startsWith('---')) {
        var reg = /---([\s\S]*?)---/;
        return trimed.replace(reg, '');
    }
    else
        return original;
};
exports.frontmatterTransformer = frontmatterTransformer;
var DefaultBlockStyles = {
    bold: '**',
    code: '```',
    italic: '*',
    strikethrough: '~~'
};
var insertTexts = {
    link: ['[', '](#url#)'],
    image: ['![](', '#url#)'],
    table: [
        '',
        '\n\n| Column 1 | Column 2 | Column 3 |\n|:--------|:--------:|--------:|\n| Text     | Text     | Text     |\n\n'
    ],
    horizontalRule: ['', '\n\n-----\n\n']
};
var getTextState = function (editor, posArg) {
    var cm = editor.getDoc();
    var pos = posArg || cm.getCursor('start');
    var stat = editor.getTokenAt(pos);
    if (!stat.type)
        return {};
    var types = stat.type.split(' ');
    var ret = {};
    var text;
    for (var _i = 0, types_1 = types; _i < types_1.length; _i++) {
        var data = types_1[_i];
        if (data === 'strong')
            ret.bold = true;
        else if (data === 'variable-2') {
            text = cm.getLine(pos.line);
            if (/^\s*\d+\.\s/.test(text))
                ret['ordered-list'] = true;
            else
                ret['unordered-list'] = true;
        }
        else if (data === 'atom')
            ret.quote = true;
        else if (data === 'em')
            ret.italic = true;
        else if (data === 'quote')
            ret.quote = true;
        else if (data === 'strikethrough')
            ret.strikethrough = true;
        else if (data === 'comment')
            ret.code = true;
        else if (data === 'link')
            ret.link = true;
        else if (data === 'tag')
            ret.image = true;
        else if (data.match(/^header(\-[1-6])?$/))
            ret[data.replace('header', 'heading')] = true;
    }
    return ret;
};
exports.getTextState = getTextState;
var toggleBlock = function (editor, type, state, startCharsArg, endCharsArg) {
    var endChars = typeof endCharsArg === 'undefined' ? startCharsArg : endCharsArg;
    var cm = editor.getDoc();
    var text;
    var start = startCharsArg;
    var end = endChars;
    var startPoint = cm.getCursor('start');
    var endPoint = cm.getCursor('end');
    if (state[type]) {
        text = cm.getLine(startPoint.line);
        start = text.slice(0, startPoint.ch);
        end = text.slice(startPoint.ch);
        if (type === 'bold') {
            start = start.replace(/(\*\*|__)(?![\s\S]*(\*\*|__))/, '');
            end = end.replace(/(\*\*|__)/, '');
        }
        else if (type === 'italic') {
            start = start.replace(/(\*|_)(?![\s\S]*(\*|_))/, '');
            end = end.replace(/(\*|_)/, '');
        }
        else if (type === 'strikethrough') {
            start = start.replace(/(\*\*|~~)(?![\s\S]*(\*\*|~~))/, '');
            end = end.replace(/(\*\*|~~)/, '');
        }
        cm.replaceRange(start + end, {
            line: startPoint.line,
            ch: 0
        }, {
            line: startPoint.line,
            ch: 99999999999999
        });
        if (type === 'bold' || type === 'strikethrough') {
            startPoint.ch -= 2;
            if (startPoint !== endPoint)
                endPoint.ch -= 2;
        }
        else if (type === 'italic') {
            startPoint.ch -= 1;
            if (startPoint !== endPoint)
                endPoint.ch -= 1;
        }
    }
    else {
        text = cm.getSelection();
        if (type === 'bold') {
            text = text.split('**').join('');
            text = text.split('__').join('');
        }
        else if (type === 'italic') {
            text = text.split('*').join('');
            text = text.split('_').join('');
        }
        else if (type === 'strikethrough')
            text = text.split('~~').join('');
        cm.replaceSelection(start + text + end);
        startPoint.ch += startCharsArg.length;
        endPoint.ch = startPoint.ch + text.length;
    }
    cm.setSelection(startPoint, endPoint);
    editor.focus();
};
var toggleBold = function (editor, state, blockStyles) {
    var block = blockStyles ? blockStyles.bold : DefaultBlockStyles.bold;
    toggleBlock(editor, 'bold', state, block);
};
exports.toggleBold = toggleBold;
var toggleItalic = function (editor, state, blockStyles) {
    var block = blockStyles ? blockStyles.italic : DefaultBlockStyles.italic;
    toggleBlock(editor, 'italic', state, block);
};
exports.toggleItalic = toggleItalic;
var toggleStrikethrough = function (editor, state, blockStyles) {
    var block = blockStyles
        ? blockStyles.strikethrough
        : DefaultBlockStyles.strikethrough;
    toggleBlock(editor, 'strikethrough', state, block);
};
exports.toggleStrikethrough = toggleStrikethrough;
var toggleHeadingBase = function (editor, direction, size) {
    var cm = editor.getDoc();
    var startPoint = cm.getCursor('start');
    var endPoint = cm.getCursor('end');
    for (var p = startPoint.line; p <= endPoint.line; p++) {
        ;
        (function (i) {
            var text = cm.getLine(i);
            var currHeadingLevel = text.search(/[^#]/);
            if (direction !== undefined) {
                if (currHeadingLevel <= 0) {
                    if (direction === 'bigger') {
                        text = '###### ' + text;
                    }
                    else {
                        text = '# ' + text;
                    }
                }
                else if (currHeadingLevel === 6 && direction === 'smaller') {
                    text = text.substr(7);
                }
                else if (currHeadingLevel === 1 && direction === 'bigger') {
                    text = text.substr(2);
                }
                else {
                    if (direction === 'bigger') {
                        text = text.substr(1);
                    }
                    else {
                        text = '#' + text;
                    }
                }
            }
            else {
                if (size === 1) {
                    if (currHeadingLevel <= 0) {
                        text = '# ' + text;
                    }
                    else if (currHeadingLevel === size) {
                        text = text.substr(currHeadingLevel + 1);
                    }
                    else {
                        text = '# ' + text.substr(currHeadingLevel + 1);
                    }
                }
                else if (size === 2) {
                    if (currHeadingLevel <= 0) {
                        text = '## ' + text;
                    }
                    else if (currHeadingLevel === size) {
                        text = text.substr(currHeadingLevel + 1);
                    }
                    else {
                        text = '## ' + text.substr(currHeadingLevel + 1);
                    }
                }
                else {
                    if (currHeadingLevel <= 0) {
                        text = '### ' + text;
                    }
                    else if (currHeadingLevel === size) {
                        text = text.substr(currHeadingLevel + 1);
                    }
                    else {
                        text = '### ' + text.substr(currHeadingLevel + 1);
                    }
                }
            }
            cm.replaceRange(text, {
                line: i,
                ch: 0
            }, {
                line: i,
                ch: 99999999999999
            });
        })(p);
    }
    editor.focus();
};
var toggleHeading = function (editor) {
    return toggleHeadingBase(editor, 'smaller');
};
exports.toggleHeading = toggleHeading;
var toggleLine = function (editor, stat, name) {
    var cm = editor.getDoc();
    var startPoint = cm.getCursor('start');
    var endPoint = cm.getCursor('end');
    var repl = {
        quote: /^(\s*)\>\s+/,
        'unordered-list': /^(\s*)(\*|\-|\+)\s+/,
        'ordered-list': /^(\s*)\d+\.\s+/
    };
    var map = {
        quote: '> ',
        'unordered-list': '* ',
        'ordered-list': '1. '
    };
    for (var p = startPoint.line; p <= endPoint.line; p++) {
        ;
        (function (i) {
            var text = cm.getLine(i);
            if (stat[name]) {
                text = text.replace(repl[name], '$1');
            }
            else {
                text = map[name] + text;
            }
            cm.replaceRange(text, {
                line: i,
                ch: 0
            }, {
                line: i,
                ch: 99999999999999
            });
        })(p);
    }
    editor.focus();
};
var toggleQuote = function (editor, state) {
    toggleLine(editor, state, 'quote');
};
exports.toggleQuote = toggleQuote;
var toggleUnorderedList = function (editor, state) {
    toggleLine(editor, state, 'unordered-list');
};
exports.toggleUnorderedList = toggleUnorderedList;
var toggleOrderedList = function (editor, state) {
    toggleLine(editor, state, 'ordered-list');
};
exports.toggleOrderedList = toggleOrderedList;
var replaceSelection = function (editor, active, startEnd, url) {
    var text;
    var start = startEnd[0];
    var end = startEnd[1];
    var cm = editor.getDoc();
    var startPoint = cm.getCursor('start');
    var endPoint = cm.getCursor('end');
    if (url) {
        end = end.replace('#url#', url);
    }
    if (active) {
        text = cm.getLine(startPoint.line);
        start = text.slice(0, startPoint.ch);
        end = text.slice(startPoint.ch);
        cm.replaceRange(start + end, {
            line: startPoint.line,
            ch: 0
        });
    }
    else {
        text = cm.getSelection();
        cm.replaceSelection(start + text + end);
        startPoint.ch += start.length;
        if (startPoint !== endPoint) {
            endPoint.ch += start.length;
        }
    }
    cm.setSelection(startPoint, endPoint);
    editor.focus();
};
var drawLink = function (editor, stat) {
    var url = 'http://';
    replaceSelection(editor, stat.link, insertTexts.link, url);
};
exports.drawLink = drawLink;
var drawImage = function (editor, stat) {
    var url = 'http://';
    replaceSelection(editor, stat.image, insertTexts.image, url);
};
exports.drawImage = drawImage;
var drawTable = function (editor, stat) {
    replaceSelection(editor, stat.table, insertTexts.table);
};
exports.drawTable = drawTable;
var toggleCodeBlock = function (editor) {
    var fenceCharsToInsertZ = DefaultBlockStyles.code;
    var insertFencingAtSelection = function (curStartArg, curEndArg, fenceCharsToInsert) {
        var cmZ = editor.getDoc();
        var startLineSel = curStartArg.line + 1;
        var endLineSel = curEndArg.line + 1;
        var selMultiZ = curStartArg.line !== curEndArg.line;
        var replStart = fenceCharsToInsert + '\n';
        var replEnd = '\n' + fenceCharsToInsert;
        if (selMultiZ) {
            endLineSel++;
        }
        if (selMultiZ && curEndArg.ch === 0) {
            replEnd = fenceCharsToInsert + '\n';
            endLineSel--;
        }
        replaceSelection(editor, false, [replStart, replEnd]);
        cmZ.setSelection({
            line: startLineSel,
            ch: 0
        }, {
            line: endLineSel,
            ch: 0
        });
    };
    var cm = editor.getDoc();
    var curStart = cm.getCursor('start');
    var curEnd = cm.getCursor('end');
    var noSelAndStartingOfLine = curStart.line === curEnd.line &&
        curStart.ch === curEnd.ch &&
        curStart.ch === 0;
    var selMulti = curStart.line !== curEnd.line;
    if (noSelAndStartingOfLine || selMulti) {
        insertFencingAtSelection(curStart, curEnd, fenceCharsToInsertZ);
    }
    else {
        replaceSelection(editor, false, ['`', '`']);
    }
};
exports.toggleCodeBlock = toggleCodeBlock;
var drawHorizontalRule = function (editor, stat) {
    replaceSelection(editor, stat.image, insertTexts.horizontalRule);
};
exports.drawHorizontalRule = drawHorizontalRule;
var cleanBlock = function (editor) {
    var cm = editor.getDoc();
    var startPoint = cm.getCursor('start');
    var endPoint = cm.getCursor('end');
    var text;
    for (var line = startPoint.line; line <= endPoint.line; line++) {
        text = cm.getLine(line);
        text = text.replace(/^[ ]*([# ]+|\*|\-|[> ]+|[0-9]+(.|\)))[ ]*/, '');
        cm.replaceRange(text, {
            line: line,
            ch: 0
        }, {
            line: line,
            ch: 99999999999999
        });
    }
};
exports.cleanBlock = cleanBlock;
//# sourceMappingURL=helpers.js.map